"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var TypeMongoDBCoreModule_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypeMongoDBCoreModule = void 0;
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const type_mongodb_1 = require("type-mongodb");
const utils_1 = require("./utils");
let TypeMongoDBCoreModule = TypeMongoDBCoreModule_1 = class TypeMongoDBCoreModule {
    constructor() {
        this.logger = new common_1.Logger('TypeMongoDBModule');
    }
    onApplicationShutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            // close all the known document managers
            yield Promise.all(utils_1.Storage.getDocumentManagers().map((dm) => new Promise((resolve, reject) => {
                if (dm) {
                    try {
                        resolve(dm.close());
                    }
                    catch (e) {
                        this.logger.error(e === null || e === void 0 ? void 0 : e.message);
                        reject(e);
                    }
                }
            })));
        });
    }
    static register(connectionName, options) {
        const providers = [
            this.createDocumentManagerOptionsProvider(connectionName, options),
            this.createDocumentManagerProvider(connectionName)
        ];
        return {
            module: TypeMongoDBCoreModule_1,
            providers,
            exports: providers
        };
    }
    static registerAsync(connectionName, options) {
        const providers = [
            ...(options.providers || []),
            ...this.createAsyncDocumentManagerProviders(connectionName, options),
            this.createDocumentManagerProvider(connectionName)
        ];
        return {
            module: TypeMongoDBCoreModule_1,
            imports: options.imports || [],
            providers,
            exports: providers
        };
    }
    static createDocumentManagerOptionsProvider(connectionName, options) {
        return {
            provide: utils_1.getDocumentManagerOptionsToken(connectionName),
            useValue: options
        };
    }
    static createDocumentManagerProvider(connectionName) {
        return {
            provide: utils_1.getDocumentManagerToken(connectionName),
            useFactory: (options, moduleRef) => __awaiter(this, void 0, void 0, function* () {
                const dm = yield type_mongodb_1.DocumentManager.create(Object.assign(Object.assign({}, options), { documents: utils_1.Storage.getDocuments(connectionName), container: {
                        get: (Repository) => __awaiter(this, void 0, void 0, function* () {
                            return moduleRef.create(Repository);
                        })
                    } }));
                utils_1.Storage.addDocumentManager(connectionName, dm);
                return dm;
            }),
            inject: [utils_1.getDocumentManagerOptionsToken(connectionName), core_1.ModuleRef]
        };
    }
    static createAsyncDocumentManagerProviders(connectionName, options) {
        if (options.useExisting || options.useFactory) {
            return [
                this.createAsyncDocumentManagerOptionsProvider(connectionName, options)
            ];
        }
        if (options.useClass) {
            return [
                this.createAsyncDocumentManagerOptionsProvider(connectionName, options),
                { provide: options.useClass, useClass: options.useClass }
            ];
        }
        throw new Error('Invalid type-mongodb async options: one of `useClass`, `useExisting` or `useFactory` should be defined.');
    }
    static createAsyncDocumentManagerOptionsProvider(connectionName, options) {
        var _a;
        if (options.useFactory) {
            return {
                provide: utils_1.getDocumentManagerOptionsToken(connectionName),
                useFactory: options.useFactory,
                inject: options.inject || []
            };
        }
        const inject = [];
        if (options.useClass || options.useExisting) {
            inject.push((_a = options.useClass) !== null && _a !== void 0 ? _a : options.useExisting);
        }
        return {
            provide: utils_1.getDocumentManagerOptionsToken(connectionName),
            useFactory: (optionsFactory) => __awaiter(this, void 0, void 0, function* () { return yield optionsFactory.createTypeMongoDBOptions(); }),
            inject
        };
    }
};
TypeMongoDBCoreModule = TypeMongoDBCoreModule_1 = __decorate([
    common_1.Global(),
    common_1.Module({})
], TypeMongoDBCoreModule);
exports.TypeMongoDBCoreModule = TypeMongoDBCoreModule;
//# sourceMappingURL=type-mongodb-core.module.js.map